@@ -79,7 +79,8 @@ typedef enum
 {
   GSD_XRANDR_BOOT_BEHAVIOUR_DO_NOTHING,
   GSD_XRANDR_BOOT_BEHAVIOUR_CLONE,
-  GSD_XRANDR_BOOT_BEHAVIOUR_DOCK
+  GSD_XRANDR_BOOT_BEHAVIOUR_DOCK,
+  GSD_XRANDR_BOOT_BEHAVIOUR_FOLLOW_LID
 } GsdXrandrBootBehaviour;
 
 typedef enum
@@ -94,7 +95,8 @@ typedef enum
 {
   GSD_WACOM_ACTION_TYPE_NONE,
   GSD_WACOM_ACTION_TYPE_CUSTOM,
-  GSD_WACOM_ACTION_TYPE_SWITCH_MONITOR
+  GSD_WACOM_ACTION_TYPE_SWITCH_MONITOR,
+  GSD_WACOM_ACTION_TYPE_HELP
 } GsdWacomActionType;
 
 typedef enum
@@ -140,6 +142,10 @@ typedef enum
   GSD_INPUT_SOURCES_SWITCHER_ALT_SHIFT,
   GSD_INPUT_SOURCES_SWITCHER_CTRL_SHIFT,
   GSD_INPUT_SOURCES_SWITCHER_ALT_CTRL,
+  GSD_INPUT_SOURCES_SWITCHER_CAPS,
+  GSD_INPUT_SOURCES_SWITCHER_SHIFT_CAPS,
+  GSD_INPUT_SOURCES_SWITCHER_ALT_CAPS,
+  GSD_INPUT_SOURCES_SWITCHER_CTRL_CAPS,
 } GsdInputSourcesSwitcher;
 
 #endif /* __gsd_enums_h__ */
