@@ -34,14 +34,14 @@
 #include <libnotify/notify.h>
 
 #include "gnome-settings-manager.h"
+#include "gnome-settings-plugin.h"
 #include "gnome-settings-profile.h"
 #include "gnome-settings-session.h"
 
-#define GSD_DBUS_NAME         "org.gnome.SettingsDaemon"
-
 #define GNOME_SESSION_DBUS_NAME      "org.gnome.SessionManager"
 #define GNOME_SESSION_CLIENT_PRIVATE_DBUS_INTERFACE "org.gnome.SessionManager.ClientPrivate"
 
+static gboolean   replace      = FALSE;
 static gboolean   debug        = FALSE;
 static gboolean   do_timed_exit = FALSE;
 static int        term_signal_pipe_fds[2];
@@ -50,6 +50,7 @@ static GnomeSettingsManager *manager = NULL;
 
 static GOptionEntry entries[] = {
         {"debug", 0, 0, G_OPTION_ARG_NONE, &debug, N_("Enable debugging code"), NULL },
+        { "replace", 'r', 0, G_OPTION_ARG_NONE, &replace, N_("Replace existing daemon"), NULL },
         { "timed-exit", 0, 0, G_OPTION_ARG_NONE, &do_timed_exit, N_("Exit after a time (for debugging)"), NULL },
         {NULL}
 };
@@ -132,6 +133,29 @@ got_client_proxy (GObject *object,
                           G_CALLBACK (client_proxy_signal_cb), manager);
 }
 
+static void
+start_settings_manager (void)
+{
+        gboolean res;
+        GError *error;
+
+        gnome_settings_profile_start ("gnome_settings_manager_new");
+        manager = gnome_settings_manager_new ();
+        gnome_settings_profile_end ("gnome_settings_manager_new");
+        if (manager == NULL) {
+                g_warning ("Unable to register object");
+                gtk_main_quit ();
+        }
+
+        error = NULL;
+        res = gnome_settings_manager_start (manager, &error);
+        if (! res) {
+                g_warning ("Unable to start: %s", error->message);
+                g_error_free (error);
+                gtk_main_quit ();
+        }
+}
+
 static void
 on_client_registered (GObject             *source_object,
                       GAsyncResult        *res,
@@ -248,25 +272,26 @@ is_program_in_path (const char *binary)
 	return TRUE;
 }
 
+static gboolean
+keyboard_plugin_is_enabled (void)
+{
+        GSettings *settings;
+        gboolean enabled;
+
+        settings = g_settings_new ("org.gnome.settings-daemon.plugins.keyboard");
+        enabled = g_settings_get_boolean (settings, "active");
+        g_object_unref (settings);
+
+        return enabled;
+}
+
 static void
 set_legacy_ibus_env_vars (GDBusProxy *proxy)
 {
-        GVariant *prop;
-        const gchar *name;
-
-        prop = g_dbus_proxy_get_cached_property (proxy, "session-name");
-        if (prop) {
-                g_variant_get (prop, "&s", &name);
-
-                if (g_strcmp0 (name, "gnome") == 0 &&
-                    is_program_in_path ("ibus-daemon")) {
-                        set_session_env (proxy, "QT_IM_MODULE", "ibus");
-                        set_session_env (proxy, "XMODIFIERS", "@im=ibus");
-                }
-
-                g_variant_unref (prop);
-        } else {
-                g_print ("failed to get SessionName\n");
+        if (is_program_in_path ("ibus-daemon") &&
+            keyboard_plugin_is_enabled ()) {
+                set_session_env (proxy, "QT_IM_MODULE", "ibus");
+                set_session_env (proxy, "XMODIFIERS", "@im=ibus");
         }
 }
 #endif
@@ -316,12 +341,12 @@ watch_for_term_signal (GnomeSettingsManager *manager)
 }
 
 static void
-set_session_over_handler (GDBusConnection *bus)
+name_acquired_handler (GDBusConnection *connection,
+                       const gchar *name,
+                       gpointer user_data)
 {
         GDBusProxy *proxy;
 
-        g_assert (bus != NULL);
-
         proxy = gnome_settings_session_get_session_proxy ();
         /* Always call this first, as Setenv can only be called before
            any client registers */
@@ -329,18 +354,11 @@ set_session_over_handler (GDBusConnection *bus)
 #ifdef HAVE_IBUS
         set_legacy_ibus_env_vars (proxy);
 #endif
+        start_settings_manager ();
         register_with_gnome_session (proxy);
         watch_for_term_signal (manager);
 }
 
-static void
-name_acquired_handler (GDBusConnection *connection,
-                       const gchar *name,
-                       gpointer user_data)
-{
-        set_session_over_handler (connection);
-}
-
 static void
 name_lost_handler (GDBusConnection *connection,
                    const gchar *name,
@@ -349,57 +367,27 @@ name_lost_handler (GDBusConnection *connection,
         /* Name was already taken, or the bus went away */
 
         g_warning ("Name taken or bus went away - shutting down");
+
+        if (manager != NULL)
+                stop_manager (manager);
+
         gtk_main_quit ();
-}
 
-static gboolean
-do_register_client (GDBusProxy *proxy)
-{
-        const char *startup_id;
-
-        g_assert (proxy != NULL);
-
-        startup_id = g_getenv ("DESKTOP_AUTOSTART_ID");
-        g_dbus_proxy_call (proxy,
-                           "RegisterClient",
-                           g_variant_new ("(ss)", "gnome-settings-daemon", startup_id ? startup_id : ""),
-                           G_DBUS_CALL_FLAGS_NONE,
-                           -1,
-                           NULL,
-                           (GAsyncReadyCallback) on_client_registered,
-                           manager);
-
-        return FALSE;
-}
-
-static void
-queue_register_client (void)
-{
-        GDBusConnection *bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
-        GDBusProxy *proxy;
-
-        if (!bus)
-                return;
-
-        proxy = gnome_settings_session_get_session_proxy ();
-        if (proxy == NULL) {
-                g_debug ("Could not connect to the Session manager");
-                return;
-        }
-
-        /* Register the daemon with gnome-session */
-        g_signal_connect (G_OBJECT (proxy), "g-signal",
-                          G_CALLBACK (on_session_over), NULL);
-
-        g_idle_add_full (G_PRIORITY_DEFAULT, (GSourceFunc) do_register_client, proxy, NULL);
 }
 
 static void
 bus_register (void)
 {
+        GBusNameOwnerFlags flags;
+
+        flags = G_BUS_NAME_OWNER_FLAGS_ALLOW_REPLACEMENT;
+
+        if (replace)
+                flags |= G_BUS_NAME_OWNER_FLAGS_REPLACE;
+
         name_id = g_bus_own_name (G_BUS_TYPE_SESSION,
                                   GSD_DBUS_NAME,
-                                  G_BUS_NAME_OWNER_FLAGS_NONE,
+                                  flags,
                                   NULL,
                                   (GBusNameAcquiredCallback) name_acquired_handler,
                                   (GBusNameLostCallback) name_lost_handler,
@@ -461,10 +449,6 @@ parse_args (int *argc, char ***argv)
 int
 main (int argc, char *argv[])
 {
-
-        gboolean              res;
-        GError               *error;
-
         gnome_settings_profile_start (NULL);
 
         bindtextdomain (GETTEXT_PACKAGE, GNOME_SETTINGS_LOCALEDIR);
@@ -474,8 +458,6 @@ main (int argc, char *argv[])
 
         parse_args (&argc, &argv);
 
-        g_type_init ();
-
         gnome_settings_profile_start ("opening gtk display");
         if (! gtk_init_check (NULL, NULL)) {
                 g_warning ("Unable to initialize GTK+");
@@ -487,26 +469,8 @@ main (int argc, char *argv[])
 
         notify_init ("gnome-settings-daemon");
 
-        queue_register_client ();
-
         bus_register ();
 
-        gnome_settings_profile_start ("gnome_settings_manager_new");
-        manager = gnome_settings_manager_new ();
-        gnome_settings_profile_end ("gnome_settings_manager_new");
-        if (manager == NULL) {
-                g_warning ("Unable to register object");
-                goto out;
-        }
-
-        error = NULL;
-        res = gnome_settings_manager_start (manager, &error);
-        if (! res) {
-                g_warning ("Unable to start: %s", error->message);
-                g_error_free (error);
-                goto out;
-        }
-
         if (do_timed_exit) {
                 g_timeout_add_seconds (30, (GSourceFunc) timed_exit_cb, NULL);
         }
@@ -515,7 +479,6 @@ main (int argc, char *argv[])
 
         g_debug ("Shutting down");
 
-out:
         if (name_id > 0) {
                 g_bus_unown_name (name_id);
                 name_id = 0;
