@@ -32,6 +32,7 @@
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnome-desktop/gnome-rr.h>
 
+#include "gnome-settings-plugin.h"
 #include "gnome-settings-profile.h"
 #include "gnome-settings-session.h"
 #include "gsd-color-manager.h"
@@ -47,7 +48,7 @@
 
 struct GsdColorManagerPrivate
 {
-        GnomeSettingsSession *session;
+        GDBusProxy      *session;
         CdClient        *client;
         GSettings       *settings;
         GcmProfileStore *profile_store;
@@ -55,7 +56,7 @@ struct GsdColorManagerPrivate
         GnomeRRScreen   *x11_screen;
         GHashTable      *edid_cache;
         GdkWindow       *gdk_window;
-        GnomeSettingsSessionState session_state;
+        gboolean         session_is_active;
         GHashTable      *device_assign_hash;
 };
 
@@ -1038,9 +1039,9 @@ gcm_session_use_output_profile_for_screen (GsdColorManager *manager,
 #define CD_PROFILE_METADATA_SCREEN_BRIGHTNESS		"SCREEN_brightness"
 #endif
 
-#define GSD_DBUS_SERVICE		"org.gnome.SettingsDaemon"
-#define GSD_DBUS_INTERFACE_POWER_SCREEN	"org.gnome.SettingsDaemon.Power.Screen"
-#define GSD_DBUS_PATH_POWER		"/org/gnome/SettingsDaemon/Power"
+#define GSD_DBUS_NAME_POWER		GSD_DBUS_NAME ".Power"
+#define GSD_DBUS_INTERFACE_POWER_SCREEN	GSD_DBUS_BASE_INTERFACE ".Power.Screen"
+#define GSD_DBUS_PATH_POWER		GSD_DBUS_PATH "/Power"
 
 static void
 gcm_session_set_output_percentage_cb (GObject *source_object,
@@ -1072,7 +1073,7 @@ gcm_session_set_output_percentage (guint percentage)
         if (connection == NULL)
                 return;
         g_dbus_connection_call (connection,
-                                GSD_DBUS_SERVICE,
+                                GSD_DBUS_NAME_POWER,
                                 GSD_DBUS_PATH_POWER,
                                 GSD_DBUS_INTERFACE_POWER_SCREEN,
                                 "SetPercentage",
@@ -1439,15 +1440,6 @@ gcm_session_add_x11_output (GsdColorManager *manager, GnomeRROutput *output)
                              gnome_rr_output_get_is_primary (output) ?
                              (gpointer) CD_DEVICE_METADATA_OUTPUT_PRIORITY_PRIMARY :
                              (gpointer) CD_DEVICE_METADATA_OUTPUT_PRIORITY_SECONDARY);
-#endif
-#if CD_CHECK_VERSION(0,1,27)
-        /* set this so we can call the device a 'Laptop Screen' in the
-         * control center main panel */
-        if (gnome_rr_output_is_laptop (output)) {
-                g_hash_table_insert (device_props,
-                                     (gpointer) CD_DEVICE_PROPERTY_EMBEDDED,
-                                     NULL);
-        }
 #endif
         cd_client_create_device (priv->client,
                                  device_id,
@@ -2228,18 +2220,40 @@ gcm_session_sensor_removed_cb (CdClient *client,
                          CA_PROP_EVENT_DESCRIPTION, _("Color calibration device removed"), NULL);
 }
 
+static gboolean
+has_changed (char       **strv,
+	     const char  *str)
+{
+        guint i;
+        for (i = 0; strv[i] != NULL; i++) {
+                if (g_str_equal (str, strv[i]))
+                        return TRUE;
+        }
+        return FALSE;
+}
+
 static void
-gcm_session_active_changed_cb (GnomeSettingsSession *session,
-                               GParamSpec *pspec,
+gcm_session_active_changed_cb (GDBusProxy      *session,
+                               GVariant        *changed,
+                               char           **invalidated,
                                GsdColorManager *manager)
 {
-        GnomeSettingsSessionState state_new;
         GsdColorManagerPrivate *priv = manager->priv;
+        GVariant *active_v = NULL;
+        gboolean is_active;
+
+        if (has_changed (invalidated, "SessionIsActive"))
+                return;
 
         /* not yet connected to the daemon */
         if (!cd_client_get_connected (priv->client))
                 return;
 
+        active_v = g_dbus_proxy_get_cached_property (session, "SessionIsActive");
+        g_return_if_fail (active_v != NULL);
+        is_active = g_variant_get_boolean (active_v);
+        g_variant_unref (active_v);
+
         /* When doing the fast-user-switch into a new account, load the
          * new users chosen profiles.
          *
@@ -2247,15 +2261,13 @@ gcm_session_active_changed_cb (GnomeSettingsSession *session,
          * loaded, then we'll get a change from unknown to active
          * and we want to avoid reprobing the devices for that.
          */
-        state_new = gnome_settings_session_get_state (session);
-        if (priv->session_state != GNOME_SETTINGS_SESSION_STATE_UNKNOWN &&
-            state_new == GNOME_SETTINGS_SESSION_STATE_ACTIVE) {
-                g_debug ("Done switch to new account, reload devices");
+        if (is_active && !priv->session_is_active) {
+                g_warning ("Done switch to new account, reload devices");
                 cd_client_get_devices (manager->priv->client, NULL,
                                        gcm_session_get_devices_cb,
                                        manager);
         }
-        priv->session_state = state_new;
+        priv->session_is_active = is_active;
 }
 
 static void
@@ -2275,11 +2287,9 @@ gsd_color_manager_init (GsdColorManager *manager)
         priv = manager->priv = GSD_COLOR_MANAGER_GET_PRIVATE (manager);
 
         /* track the active session */
-        priv->session = gnome_settings_session_new ();
-        priv->session_state = gnome_settings_session_get_state (priv->session);
-        g_signal_connect (priv->session, "notify::state",
-                          G_CALLBACK (gcm_session_active_changed_cb),
-                          manager);
+        priv->session = gnome_settings_session_get_session_proxy ();
+        g_signal_connect (priv->session, "g-properties-changed",
+                          G_CALLBACK (gcm_session_active_changed_cb), manager);
 
         /* set the _ICC_PROFILE atoms on the root screen */
         priv->gdk_window = gdk_screen_get_root_window (gdk_screen_get_default ());
