@@ -103,7 +103,6 @@ struct GsdKeyboardManagerPrivate
         GHashTable *ibus_engines;
         GHashTable *ibus_xkb_engines;
         GCancellable *ibus_cancellable;
-        gboolean session_is_fallback;
 #endif
         gint       xkb_event_base;
         GsdNumLockState old_state;
@@ -233,7 +232,8 @@ fetch_ibus_engines_result (GObject            *object,
                                                    result,
                                                    &error);
         if (!list && error) {
-                g_warning ("Couldn't finish IBus request: %s", error->message);
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+                        g_warning ("Couldn't finish IBus request: %s", error->message);
                 g_error_free (error);
 
                 clear_ibus (manager);
@@ -290,9 +290,6 @@ maybe_start_ibus (GsdKeyboardManager *manager,
         GVariantIter iter;
         const gchar *type;
 
-        if (manager->priv->session_is_fallback)
-                return;
-
         g_variant_iter_init (&iter, sources);
         while (g_variant_iter_next (&iter, "(&s&s)", &type, NULL))
                 if (g_str_equal (type, INPUT_SOURCE_TYPE_IBUS)) {
@@ -337,7 +334,8 @@ set_ibus_engine_finish (GObject            *object,
 
         result = ibus_bus_set_global_engine_async_finish (ibus, res, &error);
         if (!result) {
-                g_warning ("Couldn't set IBus engine: %s", error->message);
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+                        g_warning ("Couldn't set IBus engine: %s", error->message);
                 g_error_free (error);
         }
 }
@@ -1032,9 +1030,6 @@ apply_input_sources_settings (GSettings          *settings,
 #ifdef HAVE_IBUS
                 IBusEngineDesc *engine_desc = NULL;
 
-                if (priv->session_is_fallback)
-                        goto exit;
-
                 if (priv->ibus_engines)
                         engine_desc = g_hash_table_lookup (priv->ibus_engines, id);
                 else
@@ -1233,10 +1228,11 @@ settings_changed (GSettings          *settings,
 	    g_strcmp0 (key, KEY_BELL_MODE) == 0) {
 		g_debug ("Bell setting '%s' changed, applying bell settings", key);
 		apply_bell (manager);
-	} else if (g_strcmp0 (key, KEY_REMEMBER_NUMLOCK_STATE) == 0 ||
-		 g_strcmp0 (key, KEY_NUMLOCK_STATE) == 0) {
-		g_debug ("Num-Lock state '%s' changed, applying num-lock state settings", key);
+	} else if (g_strcmp0 (key, KEY_REMEMBER_NUMLOCK_STATE) == 0) {
+		g_debug ("Remember Num-Lock state '%s' changed, applying num-lock settings", key);
 		apply_numlock (manager);
+	} else if (g_strcmp0 (key, KEY_NUMLOCK_STATE) == 0) {
+		g_debug ("Num-Lock state '%s' changed, will apply at next startup", key);
 	} else if (g_strcmp0 (key, KEY_REPEAT) == 0 ||
 		 g_strcmp0 (key, KEY_INTERVAL) == 0 ||
 		 g_strcmp0 (key, KEY_DELAY) == 0) {
@@ -1484,10 +1480,6 @@ maybe_create_input_sources (GsdKeyboardManager *manager)
 static gboolean
 start_keyboard_idle_cb (GsdKeyboardManager *manager)
 {
-        GDBusProxy *proxy;
-        GVariant *prop;
-        const gchar *name;
-
         gnome_settings_profile_start (NULL);
 
         g_debug ("Starting keyboard manager");
@@ -1504,18 +1496,6 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
 
         maybe_create_input_sources (manager);
 
-        proxy = gnome_settings_session_get_session_proxy ();
-        prop = g_dbus_proxy_get_cached_property (proxy, "session-name");
-        if (prop) {
-                g_variant_get (prop, "&s", &name);
-                manager->priv->session_is_fallback = g_strcmp0 (name, "gnome") == 0;
-                g_variant_unref (prop);
-        } else {
-                manager->priv->session_is_fallback = FALSE;
-                g_warning ("failed to get SessionName, assuming gnome\n");
-        }
-        g_object_unref (proxy);
-
         apply_input_sources_settings (manager->priv->input_sources_settings, NULL, 0, manager);
         /* apply current settings before we install the callback */
         g_debug ("Started the keyboard plugin, applying all settings");
