@@ -45,6 +45,7 @@
 #include <gudev/gudev.h>
 #endif
 
+#include "gnome-settings-plugin.h"
 #include "gnome-settings-session.h"
 #include "gnome-settings-profile.h"
 #include "gsd-marshal.h"
@@ -59,8 +60,6 @@
 #include <pulse/pulseaudio.h>
 #include "gvc-mixer-control.h"
 
-#define GSD_DBUS_PATH "/org/gnome/SettingsDaemon"
-#define GSD_DBUS_NAME "org.gnome.SettingsDaemon"
 #define GSD_MEDIA_KEYS_DBUS_PATH GSD_DBUS_PATH "/MediaKeys"
 #define GSD_MEDIA_KEYS_DBUS_NAME GSD_DBUS_NAME ".MediaKeys"
 
@@ -71,7 +70,7 @@
 #define CUSTOM_BINDING_SCHEMA SETTINGS_BINDING_DIR ".custom-keybinding"
 
 static const gchar introspection_xml[] =
-"<node>"
+"<node name='/org/gnome/SettingsDaemon/MediaKeys'>"
 "  <interface name='org.gnome.SettingsDaemon.MediaKeys'>"
 "    <annotation name='org.freedesktop.DBus.GLib.CSymbol' value='gsd_media_keys_manager'/>"
 "    <method name='GrabMediaPlayerKeys'>"
@@ -98,10 +97,6 @@ static const gchar introspection_xml[] =
 #define VOLUME_STEP 6           /* percents for one volume button press */
 #define MAX_VOLUME 65536.0
 
-#define GNOME_DESKTOP_INPUT_SOURCES_DIR "org.gnome.desktop.input-sources"
-#define KEY_CURRENT_INPUT_SOURCE "current"
-#define KEY_INPUT_SOURCES        "sources"
-
 #define SYSTEMD_DBUS_NAME                       "org.freedesktop.login1"
 #define SYSTEMD_DBUS_PATH                       "/org/freedesktop/login1"
 #define SYSTEMD_DBUS_INTERFACE                  "org.freedesktop.login1.Manager"
@@ -110,7 +105,7 @@ static const gchar introspection_xml[] =
 
 typedef struct {
         char   *application;
-        char   *name;
+        char   *dbus_name;
         guint32 time;
         guint   watch_id;
 } MediaPlayer;
@@ -891,6 +886,27 @@ do_execute_desktop (GsdMediaKeysManager *manager,
 	}
 }
 
+static void
+do_execute_desktop_or_desktop (GsdMediaKeysManager *manager,
+			       const char          *desktop,
+			       const char          *alt_desktop,
+			       gint64               timestamp)
+{
+        GDesktopAppInfo *app_info;
+
+        app_info = g_desktop_app_info_new (desktop);
+        if (app_info == NULL)
+                app_info = g_desktop_app_info_new (alt_desktop);
+
+        if (app_info != NULL) {
+                launch_app (G_APP_INFO (app_info), timestamp);
+                g_object_unref (app_info);
+                return;
+        }
+
+        g_warning ("Could not find application '%s' or '%s'", desktop, alt_desktop);
+}
+
 static void
 do_touchpad_osd_action (GsdMediaKeysManager *manager, gboolean state)
 {
@@ -1207,7 +1223,7 @@ free_media_player (MediaPlayer *player)
                 player->watch_id = 0;
         }
         g_free (player->application);
-        g_free (player->name);
+        g_free (player->dbus_name);
         g_free (player);
 }
 
@@ -1222,7 +1238,7 @@ static gint
 find_by_name (gconstpointer a,
               gconstpointer b)
 {
-        return strcmp (((MediaPlayer *)a)->name, b);
+        return strcmp (((MediaPlayer *)a)->dbus_name, b);
 }
 
 static gint
@@ -1247,7 +1263,7 @@ name_vanished_handler (GDBusConnection     *connection,
                 MediaPlayer *player;
 
                 player = iter->data;
-                g_debug ("Deregistering vanished %s (name: %s)", player->application, player->name);
+                g_debug ("Deregistering vanished %s (dbus_name: %s)", player->application, player->dbus_name);
                 free_media_player (player);
                 manager->priv->media_players = g_list_delete_link (manager->priv->media_players, iter);
         }
@@ -1263,7 +1279,7 @@ name_vanished_handler (GDBusConnection     *connection,
 static void
 gsd_media_keys_manager_grab_media_player_keys (GsdMediaKeysManager *manager,
                                                const char          *application,
-                                               const char          *name,
+                                               const char          *dbus_name,
                                                guint32              time)
 {
         GList       *iter;
@@ -1292,7 +1308,7 @@ gsd_media_keys_manager_grab_media_player_keys (GsdMediaKeysManager *manager,
         }
 
         watch_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
-                                     name,
+                                     dbus_name,
                                      G_BUS_NAME_WATCHER_FLAGS_NONE,
                                      NULL,
                                      (GBusNameVanishedCallback) name_vanished_handler,
@@ -1302,7 +1318,7 @@ gsd_media_keys_manager_grab_media_player_keys (GsdMediaKeysManager *manager,
         g_debug ("Registering %s at %u", application, time);
         media_player = g_new0 (MediaPlayer, 1);
         media_player->application = g_strdup (application);
-        media_player->name = g_strdup (name);
+        media_player->dbus_name = g_strdup (dbus_name);
         media_player->time = time;
         media_player->watch_id = watch_id;
 
@@ -1336,7 +1352,7 @@ gsd_media_keys_manager_release_media_player_keys (GsdMediaKeysManager *manager,
                 MediaPlayer *player;
 
                 player = iter->data;
-                g_debug ("Deregistering %s (name: %s)", application, player->name);
+                g_debug ("Deregistering %s (dbus_name: %s)", application, player->dbus_name);
                 free_media_player (player);
                 manager->priv->media_players = g_list_delete_link (manager->priv->media_players, iter);
         }
@@ -1371,7 +1387,7 @@ gsd_media_player_key_pressed (GsdMediaKeysManager *manager,
         application = player->application;
 
         if (g_dbus_connection_emit_signal (manager->priv->connection,
-                                           player->name,
+                                           player->dbus_name,
                                            GSD_MEDIA_KEYS_DBUS_PATH,
                                            GSD_MEDIA_KEYS_DBUS_NAME,
                                            "MediaPlayerKeyPressed",
@@ -1785,40 +1801,6 @@ do_keyboard_brightness_action (GsdMediaKeysManager *manager,
                            manager);
 }
 
-static void
-do_switch_input_source_action (GsdMediaKeysManager *manager,
-                               MediaKeyType         type)
-{
-        GSettings *settings;
-        GVariant *sources;
-        gint i, n;
-
-        settings = g_settings_new (GNOME_DESKTOP_INPUT_SOURCES_DIR);
-        sources = g_settings_get_value (settings, KEY_INPUT_SOURCES);
-
-        n = g_variant_n_children (sources);
-        if (n < 2)
-                goto out;
-
-        i = g_settings_get_uint (settings, KEY_CURRENT_INPUT_SOURCE);
-
-        if (type == SWITCH_INPUT_SOURCE_KEY)
-                i += 1;
-        else
-                i -= 1;
-
-        if (i < 0)
-                i = n - 1;
-        else if (i >= n)
-                i = 0;
-
-        g_settings_set_uint (settings, KEY_CURRENT_INPUT_SOURCE, i);
-
- out:
-        g_variant_unref (sources);
-        g_object_unref (settings);
-}
-
 static void
 do_custom_action (GsdMediaKeysManager *manager,
                   MediaKey            *key,
@@ -1833,8 +1815,6 @@ do_action (GsdMediaKeysManager *manager,
            MediaKeyType         type,
            gint64               timestamp)
 {
-        char *cmd;
-
         g_debug ("Launching action for key type '%d' (on device id %d)", type, deviceid);
 
         switch (type) {
@@ -1871,12 +1851,10 @@ do_action (GsdMediaKeysManager *manager,
                 do_home_key_action (manager, timestamp);
                 break;
         case SEARCH_KEY:
-                cmd = NULL;
-                if ((cmd = g_find_program_in_path ("tracker-search-tool")))
-                        do_execute_desktop (manager, "tracker-needle.desktop", timestamp);
-                else
-                        do_execute_desktop (manager, "gnome-search-tool.desktop", timestamp);
-                g_free (cmd);
+                do_execute_desktop_or_desktop (manager,
+                                               "tracker-needle.desktop",
+                                               "gnome-search-tool.desktop",
+                                               timestamp);
                 break;
         case EMAIL_KEY:
                 do_url_action (manager, "mailto", timestamp);
@@ -1912,7 +1890,7 @@ do_action (GsdMediaKeysManager *manager,
                 do_media_action (manager, timestamp);
                 break;
         case CALCULATOR_KEY:
-                do_execute_desktop (manager, "gcalctool.desktop", timestamp);
+                do_execute_desktop_or_desktop (manager, "gcalctool.desktop", "gnome-calculator.desktop", timestamp);
                 break;
         case PLAY_KEY:
                 return do_multimedia_player_action (manager, "Play");
@@ -1982,10 +1960,6 @@ do_action (GsdMediaKeysManager *manager,
         case BATTERY_KEY:
                 do_execute_desktop (manager, "gnome-power-statistics.desktop", timestamp);
                 break;
-        case SWITCH_INPUT_SOURCE_KEY:
-        case SWITCH_INPUT_SOURCE_BACKWARD_KEY:
-                do_switch_input_source_action (manager, type);
-                break;
         /* Note, no default so compiler catches missing keys */
         case CUSTOM_KEY:
                 g_assert_not_reached ();
@@ -2051,6 +2025,10 @@ filter_key_events (XEvent              *xevent,
                         case VOLUME_UP_KEY:
                         case VOLUME_DOWN_QUIET_KEY:
                         case VOLUME_UP_QUIET_KEY:
+                        case SCREEN_BRIGHTNESS_UP_KEY:
+                        case SCREEN_BRIGHTNESS_DOWN_KEY:
+                        case KEYBOARD_BRIGHTNESS_UP_KEY:
+                        case KEYBOARD_BRIGHTNESS_DOWN_KEY:
                                 /* auto-repeatable keys */
                                 if (xiev->evtype != XI_KeyPress)
                                         return GDK_FILTER_CONTINUE;
@@ -2558,9 +2536,9 @@ on_bus_gotten (GObject             *source_object,
         g_dbus_proxy_new (manager->priv->connection,
                           G_DBUS_PROXY_FLAGS_NONE,
                           NULL,
-                          "org.gnome.SettingsDaemon",
-                          "/org/gnome/SettingsDaemon/XRANDR",
-                          "org.gnome.SettingsDaemon.XRANDR_2",
+                          GSD_DBUS_NAME ".XRANDR",
+                          GSD_DBUS_PATH "/XRANDR",
+                          GSD_DBUS_BASE_INTERFACE ".XRANDR_2",
                           NULL,
                           (GAsyncReadyCallback) xrandr_ready_cb,
                           manager);
@@ -2568,9 +2546,9 @@ on_bus_gotten (GObject             *source_object,
         g_dbus_proxy_new (manager->priv->connection,
                           G_DBUS_PROXY_FLAGS_NONE,
                           NULL,
-                          "org.gnome.SettingsDaemon",
-                          "/org/gnome/SettingsDaemon/Power",
-                          "org.gnome.SettingsDaemon.Power.Screen",
+                          GSD_DBUS_NAME ".Power",
+                          GSD_DBUS_PATH "/Power",
+                          GSD_DBUS_BASE_INTERFACE ".Power.Screen",
                           NULL,
                           (GAsyncReadyCallback) power_screen_ready_cb,
                           manager);
@@ -2578,9 +2556,9 @@ on_bus_gotten (GObject             *source_object,
         g_dbus_proxy_new (manager->priv->connection,
                           G_DBUS_PROXY_FLAGS_NONE,
                           NULL,
-                          "org.gnome.SettingsDaemon",
-                          "/org/gnome/SettingsDaemon/Power",
-                          "org.gnome.SettingsDaemon.Power.Keyboard",
+                          GSD_DBUS_NAME ".Power",
+                          GSD_DBUS_PATH "/Power",
+                          GSD_DBUS_BASE_INTERFACE ".Power.Keyboard",
                           NULL,
                           (GAsyncReadyCallback) power_keyboard_ready_cb,
                           manager);
