@@ -47,6 +47,7 @@
 #include "gnome-settings-profile.h"
 #include "gsd-wacom-manager.h"
 #include "gsd-wacom-device.h"
+#include "gsd-wacom-osd-window.h"
 
 #define GSD_WACOM_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_WACOM_MANAGER, GsdWacomManagerPrivate))
 
@@ -83,6 +84,9 @@ struct GsdWacomManagerPrivate
         /* button capture */
         GSList *screens;
         int      opcode;
+
+        /* Help OSD window */
+        GtkWidget *osd_window;
 };
 
 static void     gsd_wacom_manager_class_init  (GsdWacomManagerClass *klass);
@@ -540,56 +544,10 @@ apply_stylus_settings (GsdWacomDevice *device)
 	set_pressurethreshold (device, threshold);
 }
 
-/*
- * The rule to determine the status LED to use is as follow:
- *
- * "[...] if a device has only one ring/strip, use status_led0_select;
- *  otherwise the left ring/strip is controlled by status_led1_select and
- *  the right ring/strip by status_led0_select."
- *
- * http://sourceforge.net/mailarchive/message.php?msg_id=29898591
- */
-static int
-get_led_group_id(GsdWacomDevice *device,
-		 int             group_id)
-{
-	gint num_rings;
-	gint num_strips;
-
-	num_rings = gsd_wacom_device_get_num_rings (device);
-	num_strips = gsd_wacom_device_get_num_strips (device);
-
-	/* Given group_id is in {1..4} as follow
-	 * WACOM_BUTTON_RING_MODESWITCH        => group_id == 1
-	 * WACOM_BUTTON_RING2_MODESWITCH       => group_id == 2
-	 * WACOM_BUTTON_TOUCHSTRIP_MODESWITCH  => group_id == 3
-	 * WACOM_BUTTON_TOUCHSTRIP2_MODESWITCH => group_id == 4
-	 *
-	 * see function flags_to_group() in gsd-wacom-device.c
-	 */
-
-	if ((num_rings == 1) && (group_id == 1))
-		return 0;
-
-	if ((num_strips == 1) && (group_id == 3))
-		return 0;
-
-	if ((num_rings == 2) && (group_id == 1 ||  group_id == 2))
-		return (group_id & 1);
-
-	if ((num_strips == 2) && (group_id == 3 ||  group_id == 4))
-		return (group_id & 1);
-
-	g_debug ("Unhandled number of rings/strips setup (%d ring(s), %d strip(s), mode=%d",
-		 num_rings, num_strips, group_id);
-
-	return -1;
-}
-
 static void
-set_led (GsdWacomDevice *device,
-	 int             group_id,
-	 int             index)
+set_led (GsdWacomDevice       *device,
+	 GsdWacomTabletButton *button,
+	 int                   index)
 {
 	GError *error = NULL;
 	const char *path;
@@ -604,14 +562,14 @@ set_led (GsdWacomDevice *device,
 	g_return_if_fail (index >= 1);
 
 	path = gsd_wacom_device_get_path (device);
-	status_led = get_led_group_id (device, group_id);
+	status_led = button->status_led;
 
-	if (status_led < 0) {
+	if (status_led == GSD_WACOM_NO_LED) {
 		g_debug ("Ignoring unhandled group ID %d for device %s",
-		         group_id, gsd_wacom_device_get_name (device));
+		         button->group_id, gsd_wacom_device_get_name (device));
 		return;
 	}
-	g_debug ("Switching group ID %d to index %d for device %s", group_id, index, path);
+	g_debug ("Switching group ID %d to index %d for device %s", button->group_id, index, path);
 
 	command = g_strdup_printf ("pkexec " LIBEXECDIR "/gsd-wacom-led-helper --path %s --group %d --led %d",
 				   path, status_led, index - 1);
@@ -700,6 +658,7 @@ reset_pad_buttons (GsdWacomDevice *device)
 	int nmap;
 	unsigned char *map;
 	int i, j, rc;
+	GList *buttons, *l;
 
 	/* Normal buttons */
 	xdev = open_device (device);
@@ -733,9 +692,15 @@ reset_pad_buttons (GsdWacomDevice *device)
 	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev);
 
 	/* Reset all the LEDs */
-	/* FIXME, get the number of modes somewhere else */
-	for (i = 1; i <= 4; i++)
-		set_led (device, i, 1);
+	buttons = gsd_wacom_device_get_buttons (device);
+	for (l = buttons; l != NULL; l = l->next) {
+		GsdWacomTabletButton *button = l->data;
+                if (button->type == WACOM_TABLET_BUTTON_TYPE_HARDCODED &&
+                    button->status_led != GSD_WACOM_NO_LED) {
+                        set_led (device, button, 1);
+                }
+        }
+        g_list_free (buttons);
 }
 
 static void
@@ -889,6 +854,97 @@ last_stylus_changed (GsdWacomDevice  *device,
 	apply_stylus_settings (device);
 }
 
+static void
+osd_window_destroy (GsdWacomManager *manager)
+{
+	g_return_if_fail (manager != NULL);
+
+        g_clear_pointer (&manager->priv->osd_window, gtk_widget_destroy);
+}
+
+static gboolean
+osd_window_on_key_release_event (GtkWidget   *widget,
+                                 GdkEventKey *event,
+                                 GsdWacomManager *manager)
+{
+	osd_window_destroy (manager);
+
+	return FALSE;
+}
+
+static gboolean
+osd_window_on_focus_out_event (GtkWidget *widget,
+                               GdkEvent  *event,
+                               GsdWacomManager *manager)
+{
+	/* If the OSD window loses focus, hide it */
+	osd_window_destroy (manager);
+
+	return FALSE;
+}
+
+static gboolean
+osd_window_toggle_visibility (GsdWacomManager *manager,
+                              GsdWacomDevice  *device)
+{
+	GtkWidget *widget;
+        const gchar *layout_path;
+
+	if (manager->priv->osd_window) {
+		osd_window_destroy (manager);
+		return FALSE;
+	}
+
+        layout_path = gsd_wacom_device_get_layout_path (device);
+        if (layout_path == NULL) {
+                g_warning ("Cannot display the on-screen help window as the tablet "
+                           "definition for %s is missing the layout\n"
+                           "Please consider contributing the layout for your "
+                           "tablet to libwacom at linuxwacom-devel@lists.sourceforge.net\n",
+                           gsd_wacom_device_get_name (device));
+		return FALSE;
+        }
+
+        if (g_file_test (layout_path, G_FILE_TEST_EXISTS) == FALSE) {
+                g_warning ("Cannot display the on-screen help window as the "
+                           "layout file %s cannot be found on disk\n"
+                           "Please check your libwacom installation\n",
+                           layout_path);
+		return FALSE;
+        }
+
+	widget = gsd_wacom_osd_window_new (device, NULL);
+
+	/* Connect some signals to the OSD window */
+	g_signal_connect (widget, "key-release-event",
+			  G_CALLBACK(osd_window_on_key_release_event), manager);
+	g_signal_connect (widget, "focus-out-event",
+			  G_CALLBACK(osd_window_on_focus_out_event), manager);
+	g_object_add_weak_pointer (G_OBJECT (widget), (gpointer *) &manager->priv->osd_window);
+
+	gtk_window_present (GTK_WINDOW(widget));
+	manager->priv->osd_window = widget;
+
+	return TRUE;
+}
+
+static gboolean
+osd_window_update_viewable (GsdWacomManager      *manager,
+                            GsdWacomTabletButton *button,
+                            GtkDirectionType      dir,
+                            XIEvent              *xiev)
+{
+	if (manager->priv->osd_window == NULL)
+		return FALSE;
+
+	gsd_wacom_osd_window_set_active (GSD_WACOM_OSD_WINDOW (manager->priv->osd_window),
+	                                 button,
+	                                 dir,
+	                                 xiev->evtype == XI_ButtonPress);
+
+	return TRUE;
+}
+
 static void
 device_added_cb (GdkDeviceManager *device_manager,
                  GdkDevice        *gdk_device,
@@ -1042,7 +1098,8 @@ generate_key (GsdWacomTabletButton *wbutton,
 	int                   n_keys;
 	guint                 i;
 
-	if (wbutton->type == WACOM_TABLET_BUTTON_TYPE_ELEVATOR)
+	if (wbutton->type == WACOM_TABLET_BUTTON_TYPE_STRIP ||
+	    wbutton->type == WACOM_TABLET_BUTTON_TYPE_RING)
 		str = get_elevator_shortcut_string (wbutton->settings, dir);
 	else
 		str = g_settings_get_string (wbutton->settings, KEY_CUSTOM_ACTION);
@@ -1133,6 +1190,19 @@ switch_monitor (GsdWacomDevice *device)
 	gsd_wacom_device_set_display (device, current_monitor);
 }
 
+static const char*
+get_direction_name (GsdWacomTabletButtonType type,
+                    GtkDirectionType         dir)
+{
+        if (type == WACOM_TABLET_BUTTON_TYPE_RING)
+                return (dir == GTK_DIR_UP ? " 'CCW'" : " 'CW'");
+
+        if (type == WACOM_TABLET_BUTTON_TYPE_STRIP)
+                return (dir == GTK_DIR_UP ? " 'up'" : " 'down'");
+
+        return "";
+}
+
 static GdkFilterReturn
 filter_button_events (XEvent          *xevent,
                       GdkEvent        *event,
@@ -1146,6 +1216,7 @@ filter_button_events (XEvent          *xevent,
 	int                  button;
 	GsdWacomTabletButton *wbutton;
 	GtkDirectionType      dir;
+	gboolean emulate;
 
         /* verify we have a key event */
 	if (xevent->type != GenericEvent)
@@ -1167,6 +1238,11 @@ filter_button_events (XEvent          *xevent,
 	if (gsd_wacom_device_get_device_type (device) != WACOM_TYPE_PAD)
 		return GDK_FILTER_CONTINUE;
 
+	if ((manager->priv->osd_window != NULL) &&
+	    (device != gsd_wacom_osd_window_get_device (GSD_WACOM_OSD_WINDOW(manager->priv->osd_window))))
+		/* This is a button event from another device while showing OSD window */
+		osd_window_destroy (manager);
+
 	button = xev->detail;
 
 	wbutton = gsd_wacom_device_get_button (device, button, &dir);
@@ -1179,8 +1255,7 @@ filter_button_events (XEvent          *xevent,
 	g_debug ("Received event button %s '%s'%s ('%d') on device '%s' ('%d')",
 		 xiev->evtype == XI_ButtonPress ? "press" : "release",
 		 wbutton->id,
-		 wbutton->type == WACOM_TABLET_BUTTON_TYPE_ELEVATOR ?
-		 (dir == GTK_DIR_UP ? " 'up'" : " 'down'") : "",
+		 get_direction_name (wbutton->type, dir),
 		 button,
 		 gsd_wacom_device_get_name (device),
 		 deviceid);
@@ -1188,19 +1263,36 @@ filter_button_events (XEvent          *xevent,
 	if (wbutton->type == WACOM_TABLET_BUTTON_TYPE_HARDCODED) {
 		int new_mode;
 
+		/* Update OSD window if shown */
+		if (osd_window_update_viewable (manager, wbutton, dir, xiev))
+			return GDK_FILTER_REMOVE;
+
 		/* We switch modes on key release */
 		if (xiev->evtype == XI_ButtonRelease)
 			return GDK_FILTER_REMOVE;
 
-		new_mode = gsd_wacom_device_set_next_mode (device, wbutton->group_id);
-		set_led (device, wbutton->group_id, new_mode);
+		new_mode = gsd_wacom_device_set_next_mode (device, wbutton);
+		set_led (device, wbutton, new_mode);
 		return GDK_FILTER_REMOVE;
 	}
 
+	/* Update OSD window if shown */
+	emulate = osd_window_update_viewable (manager, wbutton, dir, xiev);
+
 	/* Nothing to do */
 	if (g_settings_get_enum (wbutton->settings, KEY_ACTION_TYPE) == GSD_WACOM_ACTION_TYPE_NONE)
 		return GDK_FILTER_REMOVE;
 
+	/* Show OSD window when requested */
+	if (g_settings_get_enum (wbutton->settings, KEY_ACTION_TYPE) == GSD_WACOM_ACTION_TYPE_HELP) {
+		if (xiev->evtype == XI_ButtonRelease)
+			osd_window_toggle_visibility (manager, device);
+		return GDK_FILTER_REMOVE;
+	}
+
+	if (emulate)
+		return GDK_FILTER_REMOVE;
+
 	/* Switch monitor */
 	if (g_settings_get_enum (wbutton->settings, KEY_ACTION_TYPE) == GSD_WACOM_ACTION_TYPE_SWITCH_MONITOR) {
 		if (xiev->evtype == XI_ButtonRelease)
@@ -1381,8 +1473,25 @@ gsd_wacom_manager_stop (GsdWacomManager *manager)
         g_debug ("Stopping wacom manager");
 
         if (p->device_manager != NULL) {
+                GList *devices;
+
                 g_signal_handler_disconnect (p->device_manager, p->device_added_id);
                 g_signal_handler_disconnect (p->device_manager, p->device_removed_id);
+
+                devices = gdk_device_manager_list_devices (p->device_manager, GDK_DEVICE_TYPE_SLAVE);
+                for (l = devices; l != NULL; l = l->next) {
+                        GsdWacomDeviceType type;
+
+                        type = gsd_wacom_device_get_device_type (l->data);
+                        if (type == WACOM_TYPE_PAD) {
+                                int id;
+
+                                id = get_device_id (l->data);
+                                grab_button (id, FALSE, manager->priv->screens);
+                        }
+                }
+                g_list_free (devices);
+
                 p->device_manager = NULL;
         }
 
@@ -1394,6 +1503,8 @@ gsd_wacom_manager_stop (GsdWacomManager *manager)
 
 	for (l = p->rr_screens; l != NULL; l = l->next)
 		g_signal_handlers_disconnect_by_func (l->data, on_screen_changed_cb, manager);
+
+        g_clear_pointer (&p->osd_window, gtk_widget_destroy);
 }
 
 static void
