@@ -45,7 +45,7 @@
 
 #include <libgnome-desktop/gnome-rr-config.h>
 #include <libgnome-desktop/gnome-rr.h>
-#include <libgnome-desktop/gnome-rr-labeler.h>
+#include <libgnome-desktop/gnome-pnp-ids.h>
 
 #ifdef HAVE_WACOM
 #include <libwacom/libwacom.h>
@@ -53,6 +53,7 @@
 
 #include "gsd-enums.h"
 #include "gsd-input-helper.h"
+#include "gnome-settings-plugin.h"
 #include "gnome-settings-profile.h"
 #include "gnome-settings-session.h"
 #include "gsd-xrandr-manager.h"
@@ -71,11 +72,11 @@
 /* name of the icon files (gsd-xrandr.svg, etc.) */
 #define GSD_XRANDR_ICON_NAME "gsd-xrandr"
 
-#define GSD_DBUS_PATH "/org/gnome/SettingsDaemon"
+#define GSD_XRANDR_DBUS_NAME GSD_DBUS_NAME ".XRANDR"
 #define GSD_XRANDR_DBUS_PATH GSD_DBUS_PATH "/XRANDR"
 
 static const gchar introspection_xml[] =
-"<node>"
+"<node name='/org/gnome/SettingsDaemon/XRANDR'>"
 "  <interface name='org.gnome.SettingsDaemon.XRANDR_2'>"
 "    <annotation name='org.freedesktop.DBus.GLib.CSymbol' value='gsd_xrandr_manager_2'/>"
 "    <method name='ApplyConfiguration'>"
@@ -103,8 +104,7 @@ static const gchar introspection_xml[] =
 "  </interface>"
 "</node>";
 
-struct GsdXrandrManagerPrivate
-{
+struct GsdXrandrManagerPrivate {
         GnomeRRScreen *rw_screen;
         gboolean running;
 
@@ -113,6 +113,7 @@ struct GsdXrandrManagerPrivate
 
         GSettings       *settings;
         GDBusNodeInfo   *introspection_data;
+        guint            name_id;
         GDBusConnection *connection;
         GCancellable    *bus_cancellable;
 
@@ -1126,6 +1127,14 @@ trim_rightmost_outputs_that_dont_fit_in_framebuffer (GnomeRRScreen *rr_screen, G
         return applicable;
 }
 
+static gboolean
+follow_laptop_lid(GsdXrandrManager *manager)
+{
+        GsdXrandrBootBehaviour val;
+        val = g_settings_get_enum (manager->priv->settings, CONF_KEY_DEFAULT_MONITORS_SETUP);
+        return val == GSD_XRANDR_BOOT_BEHAVIOUR_FOLLOW_LID || val == GSD_XRANDR_BOOT_BEHAVIOUR_CLONE;
+}
+
 static GnomeRRConfig *
 make_xinerama_setup (GsdXrandrManager *manager, GnomeRRScreen *screen)
 {
@@ -1142,7 +1151,7 @@ make_xinerama_setup (GsdXrandrManager *manager, GnomeRRScreen *screen)
                 GnomeRROutputInfo *info = outputs[i];
 
                 if (is_laptop (screen, info)) {
-                        if (laptop_lid_is_closed (manager))
+                        if (laptop_lid_is_closed (manager) && follow_laptop_lid (manager))
                                 gnome_rr_output_info_set_active (info, FALSE);
                         else {
                                 gnome_rr_output_info_set_primary (info, TRUE);
@@ -1574,7 +1583,8 @@ rotate_touchscreens (GsdXrandrManager *mgr,
                         continue;
                 }
 
-                if (device_info_is_touchscreen (&device_info[i])) {
+                if (device_info_is_touchscreen (&device_info[i]) ||
+                            device_info_is_tablet (&device_info[i])) {
                         XDevice *device;
                         char c = evdev_rotations[rot_idx].axes_swap;
                         PropertyHelper axes_swap = {
@@ -1639,7 +1649,7 @@ handle_rotate_windows (GsdXrandrManager *mgr,
         GnomeRRRotation next_rotation;
         gboolean success, show_error;
 
-        g_debug ("Handling XF86RotateWindows");
+        g_debug ("Handling XF86RotateWindows with rotation %d", rotation);
 
         /* Which output? */
 
@@ -1651,7 +1661,7 @@ handle_rotate_windows (GsdXrandrManager *mgr,
                 goto out;
         }
 
-        if (rotation < 0) {
+        if (rotation <= GNOME_RR_ROTATION_NEXT) {
                 /* Which rotation? */
 
                 get_allowed_rotations_for_output (current, priv->rw_screen, rotatable_output_info, &num_allowed_rotations, &allowed_rotations);
@@ -1867,6 +1877,12 @@ apply_default_boot_configuration (GsdXrandrManager *mgr, guint32 timestamp)
         switch (boot) {
         case GSD_XRANDR_BOOT_BEHAVIOUR_DO_NOTHING:
                 return;
+        case GSD_XRANDR_BOOT_BEHAVIOUR_FOLLOW_LID:
+                if (laptop_lid_is_closed (mgr))
+                        config = make_other_setup (screen);
+                else
+                        config = make_xinerama_setup (mgr, screen);
+                break;
         case GSD_XRANDR_BOOT_BEHAVIOUR_CLONE:
                 config = make_clone_setup (mgr, screen);
                 break;
@@ -1893,7 +1909,9 @@ apply_stored_configuration_at_startup (GsdXrandrManager *manager, guint32 timest
         gboolean success;
         char *backup_filename;
         char *intended_filename;
+        GnomePnpIds *pnp_ids;
 
+        pnp_ids = gnome_pnp_ids_new ();
         backup_filename = gnome_rr_config_get_backup_filename ();
         intended_filename = gnome_rr_config_get_intended_filename ();
 
@@ -1933,6 +1951,7 @@ apply_stored_configuration_at_startup (GsdXrandrManager *manager, guint32 timest
         success = apply_intended_configuration (manager, intended_filename, timestamp);
 
 out:
+	g_object_unref (pnp_ids);
 
         if (my_error)
                 g_error_free (my_error);
@@ -1993,6 +2012,8 @@ power_client_changed_cb (UpClient *client, gpointer data)
 
         if (is_closed != priv->laptop_lid_is_closed) {
                 priv->laptop_lid_is_closed = is_closed;
+                if (!follow_laptop_lid(manager))
+                        return;
 
                 /* Refresh the RANDR state.  The lid just got opened/closed, so we can afford to
                  * probe the outputs right now.  It will also help the case where we can't detect
@@ -2108,26 +2129,11 @@ gsd_xrandr_manager_stop (GsdXrandrManager *manager)
         log_close ();
 }
 
-static GObject *
-gsd_xrandr_manager_constructor (GType                  type,
-                              guint                  n_construct_properties,
-                              GObjectConstructParam *construct_properties)
-{
-        GsdXrandrManager      *xrandr_manager;
-
-        xrandr_manager = GSD_XRANDR_MANAGER (G_OBJECT_CLASS (gsd_xrandr_manager_parent_class)->constructor (type,
-                                                                                                      n_construct_properties,
-                                                                                                      construct_properties));
-
-        return G_OBJECT (xrandr_manager);
-}
-
 static void
 gsd_xrandr_manager_class_init (GsdXrandrManagerClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-        object_class->constructor = gsd_xrandr_manager_constructor;
         object_class->finalize = gsd_xrandr_manager_finalize;
 
         g_type_class_add_private (klass, sizeof (GsdXrandrManagerPrivate));
@@ -2145,14 +2151,17 @@ gsd_xrandr_manager_init (GsdXrandrManager *manager)
 static void
 gsd_xrandr_manager_finalize (GObject *object)
 {
-        GsdXrandrManager *xrandr_manager;
+        GsdXrandrManager *manager;
 
         g_return_if_fail (object != NULL);
         g_return_if_fail (GSD_IS_XRANDR_MANAGER (object));
 
-        xrandr_manager = GSD_XRANDR_MANAGER (object);
+        manager = GSD_XRANDR_MANAGER (object);
 
-        g_return_if_fail (xrandr_manager->priv != NULL);
+        g_return_if_fail (manager->priv != NULL);
+
+        if (manager->priv->name_id != 0)
+                g_bus_unown_name (manager->priv->name_id);
 
         G_OBJECT_CLASS (gsd_xrandr_manager_parent_class)->finalize (object);
 }
@@ -2256,6 +2265,14 @@ on_bus_gotten (GObject             *source_object,
                                                    NULL,
                                                    NULL);
         }
+
+        manager->priv->name_id = g_bus_own_name_on_connection (connection,
+                                                               GSD_XRANDR_DBUS_NAME,
+                                                               G_BUS_NAME_OWNER_FLAGS_NONE,
+                                                               NULL,
+                                                               NULL,
+                                                               NULL,
+                                                               NULL);
 }
 
 static void
